<?xml version="1.0"?>
<!-- in case of any change in xsl/xml file modify all connected xsl and xml files from this package  -->
<xsl:stylesheet version="1.1" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:saxon="http://icl.com/saxon" xmlns:textutil="textutil" xmlns:graph="graph" xmlns:results="results" extension-element-prefixes="saxon">

    <!-- ================ print functional menu ======================= -->
    <xsl:template name="display_functional_menu">
           <xsl:param name="auth"/>
           
        <xsl:variable name="href" saxon:assignable="yes"/>
        <xsl:if test="not($auth)">
        <saxon:assign name="href">#func</saxon:assign>
        </xsl:if>

        <xsl:if test="($functional > 0) or ($change_advisor > 0)">
          <xsl:variable name="caption" saxon:assignable="yes"/>
            <xsl:choose>
              <xsl:when test="($toolName='Virtualize')">
                <saxon:assign name="caption" select="$RES_PARAMETER_PROVISIONING_ACTIONS"/>
              </xsl:when>
              <xsl:otherwise>
                <saxon:assign name="caption" select="$RES_PARAMETER_FUNCTIONAL_TESTS"/>
              </xsl:otherwise>
            </xsl:choose>
          <xsl:call-template name="display_linked_menu_item">
            <xsl:with-param name="text" select="$caption" />
            <xsl:with-param name="link" select="$href" />
          </xsl:call-template>
        </xsl:if>
    </xsl:template>
        
    <!-- =================  print functional header ============================ -->
    <xsl:template name="func_header">
        <xsl:param name="isFunctional" />

        <table width="{$width}"  border="0"
            bordercolor="{$func_bdrcolor}" cellpadding="0" cellspacing="0">
            <xsl:if test="($report_format_mode='pdf')">
               <xsl:attribute name="cols">15% 70% 15%</xsl:attribute>
            </xsl:if>
            <tr>
                <td width="15%"></td>
                <td align="center">
                   <div class="header">
                       <xsl:choose>
                           <xsl:when test="($isFunctional) = 'false'">
                               <xsl:value-of select="$RES_PARAMETER_CHANGE_IMPACT"/>
                           </xsl:when>
                           <xsl:when test="($toolName='Virtualize')">
                               <xsl:value-of select="$RES_PARAMETER_PROVISIONING_ACTIONS"/>
                           </xsl:when>
                           <xsl:otherwise>
                               <xsl:value-of select="$RES_PARAMETER_FUNCTIONAL_TESTS" />
                           </xsl:otherwise>
                       </xsl:choose>
                   </div>
                </td>
                <td width="15%" align="right" valign="bottom">
                </td>
            </tr>
        </table>
    </xsl:template>
        
    <!-- =================  func_authors_summary ============================ -->
    <xsl:template name="func_authors_summary">
        <xsl:param name="isFunctional"/>
        <xsl:call-template name="authors_header"/>
        <table align="center" class="singleborder" width="{$authors_width}" cellpadding='0' cellspacing='0'>
            <xsl:if test="($report_format_mode='pdf')">
                <xsl:attribute name="cols">70% 15% 15%</xsl:attribute>
            </xsl:if>
            <tr bgcolor="{$func_bcolor}">
                <td class="singleborder right_separator" width="70%" align="center">
                    <b><xsl:value-of select="$RES_PARAMETER_AUTHOR" /></b>
                </td>
                <td class="singleborder" width="15%" align="center">
                    <b><xsl:choose>
                        <xsl:when test="($isFunctional) = 'true'">
                            <xsl:value-of select="$RES_PARAMETER_FINDINGS" />
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$RES_PARAMETER_CHANGES" />
                        </xsl:otherwise>
                    </xsl:choose></b>
                    <!--  NYI These options are not yet supported
                        <xsl:value-of select="$RES_PARAMETER_SUPPRESSED"/>
                        <xsl:value-of select="$RES_PARAMETER_TOTAL_RECOMMENDED"/> -->
                </td>
                <td class="singleborder" width="15%" align="center">
                    <b><xsl:choose>
                        <xsl:when test="($isFunctional) = 'true'">
                            <xsl:value-of select="$RES_PARAMETER_FAILED_TESTS"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$RES_PARAMETER_CHANGED_TESTS"/>
                        </xsl:otherwise>
                    </xsl:choose></b>
                        
                </td>
            </tr>
            <xsl:call-template name="func_author_list">
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            </xsl:call-template>
        </table>
    </xsl:template>
    
    <!-- ===================== get complete name for test ========= -->
    <xsl:template name="func_test_name">
        <xsl:param name="includePath"/>
        <xsl:param name="name"/>
        <xsl:param name="testID"/>
        <xsl:param name="testNode"/>
        <xsl:choose>
            <xsl:when test="($includePath)">
                <xsl:call-template name="parent_path_to_test">
                    <xsl:with-param name="dirty_path" select="($testID)" />
                </xsl:call-template>
                <xsl:variable name="pathToTest">
                    <xsl:call-template name="path_to_test">
                        <xsl:with-param name="current_node" select="($testNode)"/>
                    </xsl:call-template>
                </xsl:variable>
                <xsl:if test="not(($pathToTest) = '')">
                    - <xsl:value-of select="($pathToTest)"/>
                </xsl:if>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="($name)"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- ===================== get parent path for test =========== -->
    <xsl:template name="parent_path_to_test">
        <xsl:param name="dirty_path"/>
        <xsl:variable name="trim_start" select="substring-after(($dirty_path),':///')"/>
        <xsl:variable name="clean_path" select="substring-before(($trim_start),'#')" saxon:assignable="yes"/>

        <xsl:if test="not(string-length($clean_path) > 0)">
            <saxon:assign name="clean_path" select="substring-before(($trim_start),'.')"/>
        </xsl:if>
        <xsl:value-of select="($clean_path)"/>
    </xsl:template>

    <!-- =================  get path to test ================== -->
    <xsl:template name="path_to_test">
        <xsl:param name="current_node"/>
        <xsl:variable name="path" saxon:assignable="yes"/>
        
        <xsl:for-each select="($current_node)">
            <xsl:if test="(name() = 'TestSuite' or name() = 'Test') and contains(@id, '#')">
                <xsl:variable name="parent_path">
                    <xsl:call-template name="path_to_test">
                        <xsl:with-param name="current_node" select=".."/>
                    </xsl:call-template>
                </xsl:variable>
                <xsl:choose>
                    <xsl:when test="($parent_path) = ''">
                        <saxon:assign name="path" select="@name"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <saxon:assign name="path" select="concat(($parent_path),'/',@name)"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:if>
        </xsl:for-each>
        <xsl:value-of select="($path)"/>
    </xsl:template>
    
    <!-- =================  get total failed tests for a test suite for an author ====== -->
    <xsl:template name="total_failed_sub_tests">
        <xsl:param name="suite"/>
        <xsl:param name="auth"/>
        <xsl:param name="isFunctional"/>
        
        <xsl:variable name="success">
            <xsl:call-template name="total_success_sub_tests">
                <xsl:with-param name="suite" select="($suite)"/>
                <xsl:with-param name="auth" select="($auth)"/>
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="total">
            <xsl:call-template name="total_sub_tests">
                <xsl:with-param name="suite" select="($suite)"/>
                <xsl:with-param name="isFunctional" select="$isFunctional"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="failed" select="($total) - ($success)"/>
        <xsl:value-of select="($failed)"/>
    </xsl:template>
    
    <!-- =================  get total tests for a test suite  =============== -->
    <xsl:template name="total_sub_tests">
        <xsl:param name="suite"/>
        <xsl:param name="isFunctional"/>
        
        <xsl:variable name="test_count" saxon:assignable="yes">0</xsl:variable>
        <xsl:choose>
            <xsl:when test="($isFunctional) = 'true'">
                <saxon:assign name="test_count" select="$suite/@total"/>
            </xsl:when>
            <xsl:otherwise>
                <saxon:assign name="test_count" select="$suite/@changeTotal"/>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:if test="string-length($test_count) = 0">
          <saxon:assign name="test_count" select="0"/>
        </xsl:if>
        <xsl:value-of select="($test_count)"/>
    </xsl:template>

    <!-- =================  get total succeeded tests for a test suite for an author ======= -->
    <xsl:template name="total_success_sub_tests">
        <xsl:param name="suite"/>
        <xsl:param name="auth"/>
        <xsl:param name="isFunctional"/>
    
        <xsl:variable name="test_count" saxon:assignable="yes">0</xsl:variable>
        <xsl:choose>
            <xsl:when test="($auth) = ''">
                <xsl:choose>
                    <xsl:when test="($isFunctional) = 'true'">
                            <xsl:if test="$suite/@pass">
                                <saxon:assign name="test_count" select="$suite/@pass"/>
                            </xsl:if>
                    </xsl:when>
                    <xsl:otherwise>
                        <saxon:assign name="test_count" select="$suite/@changePass"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
            <!-- not implemented -->
               <saxon:assign name="test_count" select="-20"/>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:if test="string-length($test_count) = 0">
          <saxon:assign name="test_count" select="0"/>
        </xsl:if>
        <xsl:value-of select="($test_count)"/>
    </xsl:template>
    
    <!-- =================  get total tests ================== -->
    <xsl:template name="total_tests">
        <xsl:param name="isFunctional"/>
        <xsl:variable name="test_count" saxon:assignable="yes">0</xsl:variable>
        <xsl:choose>
            <xsl:when test="($isFunctional) = 'true'">
                <saxon:assign name="test_count" select="/ResultsSession/Exec/ExecutedTestsDetails/Total/@total"/>
            </xsl:when>
            <xsl:otherwise>
                <saxon:assign name="test_count" select="/ResultsSession/Exec/ExecutedTestsDetails/Total/@changeTotal"/>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:if test="string-length($test_count) = 0">
          <saxon:assign name="test_count" select="0"/>
        </xsl:if>
        <xsl:value-of select="($test_count)"/>
    </xsl:template>

    <!-- =================  get total failed tests for an author =========== -->
    <xsl:template name="total_failed_tests">
        <xsl:param name="auth"/>
        <xsl:param name="isFunctional"/>

        <xsl:variable name="test_count" saxon:assignable="yes">0</xsl:variable>
        <xsl:choose>
            <xsl:when test="($auth) = ''">
                <xsl:choose>
                    <xsl:when test="($isFunctional) = 'true'">
                        <saxon:assign name="test_count" select="/ResultsSession/Exec/ExecutedTestsDetails/Total/@fail"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <saxon:assign name="test_count" select="/ResultsSession/Exec/ExecutedTestsDetails/Total/@change"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
            <!-- not implemented -->
               <saxon:assign name="test_count" select="-20"/>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:if test="string-length($test_count) = 0">
          <saxon:assign name="test_count" select="0"/>
        </xsl:if>
        <xsl:value-of select="($test_count)"/>
    </xsl:template>
    
    <!-- =================  get total successful tests for an author ======= -->
    <xsl:template name="total_successful_tests">
        <xsl:param name="auth"/>
        <xsl:param name="isFunctional"/>

        <xsl:variable name="test_count" saxon:assignable="yes">0</xsl:variable>
        <xsl:choose>
            <xsl:when test="($auth) = ''">
                <xsl:choose>
                    <xsl:when test="($isFunctional) = 'true'">
                        <saxon:assign name="test_count" select="/ResultsSession/Exec/ExecutedTestsDetails/Total/@pass"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <saxon:assign name="test_count" select="/ResultsSession/Exec/ExecutedTestsDetails/Total/@changePass"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
            <!-- not implemented -->
               <saxon:assign name="test_count" select="-20"/>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:if test="string-length($test_count) = 0">
          <saxon:assign name="test_count" select="0"/>
        </xsl:if>
        <xsl:value-of select="($test_count)"/>
    </xsl:template>

    <!-- =================  print functional test summary statistics ============= -->

    <xsl:template name="func_tests_summary_stats">
        <xsl:param name="auth"/>
        <xsl:param name="isFunctional"/>

        <xsl:variable name="label" saxon:assignable="yes"></xsl:variable>

        <xsl:if test="string-length($auth) > 0">

          <xsl:if test="($isFunctional) = 'true'">
             <xsl:variable name="author_func_violations"
               select="count(/ResultsSession/Exec/ExecViols/*[not(local-name() = 'ChangeImpact')][@auth=($auth) and not(@supp='true')])" />
             <saxon:assign name="label" select="concat($RES_PARAMETER_TOTAL_FINDINGS,': ',$author_func_violations)"/>
          </xsl:if>    
        </xsl:if>

        <xsl:if test="string-length($auth) = 0">
          <xsl:variable name="totalTests">
            <xsl:call-template name="total_tests">
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:variable name="failedTests">
            <xsl:call-template name="total_failed_tests">
                <xsl:with-param name="auth" select="($auth)"/>
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            </xsl:call-template>
          </xsl:variable>

          <saxon:assign name="label" select="concat($RES_PARAMETER_TEST_FAILURES,': ',$failedTests,'/',$totalTests)"/>
        </xsl:if>

        <xsl:value-of select="$label"/>

    </xsl:template>

    <!-- =================  print execution summary charts ============= -->

    <xsl:template name="func_summary_charts">
        <xsl:param name="auth"/>

        <table width='{$width}' cellpadding='0' cellspacing='0'>
          <xsl:if test="($report_format_mode='pdf')">
          <xsl:choose>
               <xsl:when test="($apicoverage > 0) and ($change_advisor > 0)">
                  <xsl:attribute name="cols">33% 33% 34%</xsl:attribute>
               </xsl:when>
               <xsl:when test="(($apicoverage > 0) and not($change_advisor > 0))">
                  <xsl:attribute name="cols">50% 50%</xsl:attribute>
               </xsl:when>
               <xsl:when test="(not($apicoverage > 0) and ($change_advisor > 0) and ($functional > 0))">
                  <xsl:attribute name="cols">50% 50%</xsl:attribute>
               </xsl:when>
               <xsl:otherwise>
                    <xsl:attribute name="cols">100%</xsl:attribute>
               </xsl:otherwise>
          </xsl:choose>

          </xsl:if>
          <tr>
             <xsl:if test="not($report_format_mode='pdf')">
                <td width="25%"></td>
             </xsl:if>

             <xsl:if test="($apicoverage > 0)">
                <td>
                  <xsl:call-template name="apicoverage_chart"/>
                </td>
             </xsl:if>

             <xsl:if test="($functional > 0)">
               <xsl:call-template name="graphs">
                 <xsl:with-param name="auth" select="($auth)"/>
                 <xsl:with-param name="isFunctional" select="'true'"/>
               </xsl:call-template> 
             </xsl:if> 

             <xsl:if test="($change_advisor > 0)">
               <xsl:call-template name="graphs">
                 <xsl:with-param name="auth" select="($auth)"/>
                 <xsl:with-param name="isFunctional" select="'false'"/>
               </xsl:call-template> 
             </xsl:if>

             <xsl:if test="not($report_format_mode='pdf')">
                <td width="25%"></td>
             </xsl:if>
         </tr>
       </table>
    </xsl:template>

    <!-- =================  print summary charts ============= -->

    <xsl:template name="graphs">
        <xsl:param name="auth"/>
        <xsl:param name="isFunctional"/>

        <xsl:variable name="total" saxon:assignable="yes">
            <xsl:call-template name="total_tests">
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="pass" saxon:assignable="yes">
            <xsl:call-template name="total_successful_tests">
                <xsl:with-param name="auth" select="($auth)"/>
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="failed" saxon:assignable="yes">
            <xsl:call-template name="total_failed_tests">
                <xsl:with-param name="auth" select="($auth)"/>
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            </xsl:call-template>
        </xsl:variable>

        <xsl:if test="string-length($total) = 0">
           <saxon:assign name="total" select="0"/>
        </xsl:if>
        <xsl:if test="string-length($pass) = 0">
           <saxon:assign name="pass" select="0"/>
        </xsl:if>
        <xsl:if test="string-length($failed) = 0">
           <saxon:assign name="failed" select="0"/>
        </xsl:if>
        <!-- 
        <xsl:variable name="url" select="$currTagUrl" saxon:assignable="yes"/>
                            <xsl:if test="contains($url, '[%ID%]')">
                                <saxon:assign name="url"><xsl:value-of select="substring-before($url,'[%ID%]')"/><xsl:value-of select="@id"/><xsl:value-of select="substring-after($url,'[%ID%]')"/></saxon:assign>
                            </xsl:if>
         -->
        
        <xsl:variable name="bottom_text" select="concat($RES_PARAMETER_TEST_FAILURES,': ',$failed,'/',$total)" saxon:assignable="yes"/>
        <xsl:variable name="img_name" saxon:assignable="yes">func_summary_</xsl:variable>
        <xsl:variable name="item_names" saxon:assignable="yes"><xsl:value-of select="$RES_PARAMETER_FAILED_TEST_CASES"/>,<xsl:value-of select="$RES_PARAMETER_TESTS"/></xsl:variable>
          <xsl:if test="($isFunctional = 'false')">
            <saxon:assign name="bottom_text"><xsl:value-of select="concat($RES_PARAMETER_CHANGED_TESTS,': ',$failed,'/',$total)"/></saxon:assign>
            <saxon:assign name="img_name">change_summary_</saxon:assign>
            <saxon:assign name="item_names"><xsl:value-of select="$RES_PARAMETER_CHANGED_TESTS"/>,<xsl:value-of select="$RES_PARAMETER_ASSETS"/></saxon:assign>
          </xsl:if>

          <td>

            <xsl:variable name="pass_tests_percent" select="concat(textutil:getPercent($pass,$failed),'%')" saxon:assignable="yes"/>
            <xsl:if test="($isFunctional) = 'false'">
              <saxon:assign name="pass_tests_percent"><xsl:value-of select="concat(textutil:getPercent($failed,$pass),'%')"/></saxon:assign>
            </xsl:if>

            <xsl:call-template name="generate_ring_chart">
              <xsl:with-param name="title">
                <xsl:choose>
                  <xsl:when test="($isFunctional) = 'true'">
                    <xsl:value-of select="$RES_PARAMETER_TESTS"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of select="$RES_PARAMETER_CHANGE_IMPACT" />
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:with-param>
              <xsl:with-param name="central_text"><xsl:value-of select="$pass_tests_percent"/></xsl:with-param>
              <xsl:with-param name="bottom_text"><xsl:value-of select="$bottom_text"/></xsl:with-param>
               <xsl:with-param name="item_names"><xsl:value-of select="$RES_PARAMETER_FAILED_TEST_CASES"/>,<xsl:value-of select="$RES_PARAMETER_TESTS"/></xsl:with-param> 

              <xsl:with-param name="item_colors">
                <xsl:choose>
                  <xsl:when test="($isFunctional) = 'true'">
                    <xsl:value-of 
                      select="$tests_failed_color"/>,<xsl:value-of
                      select="$tests_passed_color"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of 
                      select="$covered_color"/>,<xsl:value-of
                      select="$not_covered_color"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:with-param>

              <xsl:with-param name="item_data"><xsl:value-of
                      select="$failed"/>,<xsl:value-of
                      select="$pass"/></xsl:with-param>
              <xsl:with-param name="img_name" select="concat($img_name,$org_time)"/>
            </xsl:call-template> 
          </td>
    </xsl:template>

    <!-- =================  print functional test summary info ============= -->

    <xsl:template name="func_tests_summary">
        <xsl:param name="auth"/>
        <xsl:param name="isFunctional"/>

        <xsl:variable name="totalTests">
            <xsl:call-template name="total_tests">
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="successfulTests">
            <xsl:call-template name="total_successful_tests">
                <xsl:with-param name="auth" select="($auth)"/>
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="failedTests">
            <xsl:call-template name="total_failed_tests">
                <xsl:with-param name="auth" select="($auth)"/>
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="topTest" select="/ResultsSession/Exec/ExecutedTestsDetails/Total/Project//TestSuite[contains(@id, '#') and not(contains(substring-after(@id,'#'), '#'))]" />
        <xsl:variable name="numberRootTestSuites" select="count($topTest)"/>
        <xsl:variable name="background">
            <xsl:choose>
                <xsl:when test="($failedTests) > 0">errorsingleborder</xsl:when>
                <xsl:otherwise>singleborder</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
                
        <xsl:variable name="hasTestsWithAssociations" select="count(/ResultsSession/Exec/ExecutedTestsDetails//Test/assoc) > 0"/>
        <xsl:variable name="hasTestsWithComments" select="count(/ResultsSession/Exec/ExecutedTestsDetails//Test/Detail[@msg != '']) > 0"/>
        <xsl:variable name="hasTestsWithScreenshots" select="count(/ResultsSession/Exec/ExecutedTestsDetails//Test//Screenshot) > 0"/>
        <xsl:variable name="testsHaveDetails" select="(($hasTestsWithAssociations) or ($hasTestsWithComments) or ($hasTestsWithScreenshots)) and (($isFunctional) = 'true')" />
            
        <xsl:variable name="hasFailedTestsWithAssociations" select="count(/ResultsSession/Exec/ExecutedTestsDetails//Test[@fail != '0']/assoc) > 0"/>
        <xsl:variable name="hasFailedTestsWithComments" select="count(/ResultsSession/Exec/ExecutedTestsDetails//Test/Detail[@msg != '' and @fail != '0']) > 0"/>
        <xsl:variable name="hasFailedTestsWithScreenshots" select="count(/ResultsSession/Exec/ExecutedTestsDetails//Test[@fail != '0']//Screenshot) > 0"/>
        <xsl:variable name="failedTestsHaveDetails" select="($hasFailedTestsWithAssociations) or ($hasFailedTestsWithComments) or ($hasFailedTestsWithScreenshots)" />
            
        <xsl:variable name="hasTestsToShow" select="(($report.failed_tests_only) = 'false') or ($failedTestsHaveDetails)" />
        <xsl:variable name="useExpandCollapse" select="($testsHaveDetails) and (($report.test_suites_only) = 'false') and ($hasTestsToShow) and (not(($report_type) = ($email_report)))" />
        
        <xsl:call-template name="summary_header">
            <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            <xsl:with-param name="useExpandCollapse" select="($useExpandCollapse)"/>
        </xsl:call-template>
        
        <table class="singleborder" width="{$width}"
            cellpadding="0" cellspacing="0">
            <xsl:if test="($report_format_mode='pdf')">                
                <xsl:if test="($report.test_suites_only) = 'true'">
                    <xsl:attribute name="cols">48% 13% 13% 13% 13%</xsl:attribute>
                </xsl:if>
                <xsl:if test="($report.test_suites_only) = 'false'">
                    <xsl:attribute name="cols">45% 11% 11% 11% 11% 11%</xsl:attribute>
                </xsl:if>
            </xsl:if>
           
            <tr>
                <td class="singleborder right_separator" bgcolor="{$func_bcolor}" width="45%" align="center">
                    <xsl:choose>
                        <xsl:when test="($report.test_suites_only) = 'false'">
                            <b><xsl:choose>
                                <xsl:when test="($isFunctional) = 'true'">
                                    <xsl:value-of select="$RES_PARAMETER_NAME" />
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of select="$RES_PARAMETER_IMPACTED" />
                                </xsl:otherwise>
                            </xsl:choose></b>
                        </xsl:when>
                        <xsl:otherwise>
                            <b><xsl:value-of select="$RES_PARAMETER_NAME2" /></b>
                        </xsl:otherwise>
                    </xsl:choose>
                </td>
                <td class="singleborder" bgcolor="{$func_bcolor}" align="center" width="11%">
                    <b><xsl:choose>
                        <xsl:when test="($isFunctional) = 'true'">
                            <xsl:value-of select="$RES_PARAMETER_FAILED_TESTS" />
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$RES_PARAMETER_CHANGED_TESTS" />
                        </xsl:otherwise>
                    </xsl:choose></b>
                </td>
                <td class="singleborder" bgcolor="{$func_bcolor}" align="center" width="11%">
                    <b><xsl:choose>
                        <xsl:when test="($isFunctional) = 'true'">
                            <xsl:value-of select="$RES_PARAMETER_SUCCESSFUL_TESTS" />
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$RES_PARAMETER_UNCHANGED_TESTS" />
                        </xsl:otherwise>
                    </xsl:choose></b>
                </td>
                <td class="singleborder" bgcolor="{$func_bcolor}" align="center" width="11%">
                    <b><xsl:choose>
                        <xsl:when test="($isFunctional) = 'true'">
                            <xsl:value-of select="$RES_PARAMETER_TOTAL_TESTS" />
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$RES_PARAMETER_TOTAL_ASSETS" />
                        </xsl:otherwise>
                    </xsl:choose></b>
                </td>
                <td class="singleborder" bgcolor="{$func_bcolor}" align="center" width="11%">
                    <b><xsl:choose>
                        <xsl:when test="($isFunctional) = 'true'">
                            <xsl:value-of select="$RES_PARAMETER_SUCCESS_PERCENT"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$RES_PARAMETER_IMPACT_PERCENT"/>
                        </xsl:otherwise>
                    </xsl:choose></b>
                </td>
                <xsl:if test="($report.test_suites_only) = 'false'">
                    <td class="singleborder" bgcolor="{$func_bcolor}" align="center" width="11%">
                        <b><xsl:choose>
                        <xsl:when test="($isFunctional) = 'true'">
                            <xsl:value-of select="$RES_PARAMETER_TOTAL_FINDINGS"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$RES_PARAMETER_TOTAL_CHANGES"/>
                        </xsl:otherwise>
                    </xsl:choose></b>
                    </td>
                </xsl:if>
            </tr>
            <xsl:if test="($numberRootTestSuites > 1) or ($report.test_suites_only = 'false')">
                <tr class="{$background}">
                    <td align="left" width="45%" class="singleborder right_separator">
                        <b><xsl:value-of select="$RES_PARAMETER_TOTAL"/></b>
                    </td>
                    <td width="11%" align="center" class="singleborder">
                        <xsl:value-of select="$failedTests" /><xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                    </td>
                    <td width="11%" align="center" class="singleborder">
                        <xsl:value-of select="$successfulTests" /><xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                    </td>
                    <td width="11%" align="center" class="singleborder">
                        <xsl:value-of select="$totalTests"/><xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                    </td>
                    <td width="11%" align="center" class="singleborder">
                        <xsl:variable name="numerator">
                            <xsl:choose>
                                <xsl:when test="($isFunctional) = 'true'">
                                    <xsl:value-of select="($successfulTests)"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of select="($failedTests)"/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        <xsl:call-template name="get_percent">
                            <xsl:with-param name="numerator" select="($numerator)"/>
                            <xsl:with-param name="denominator" select="($totalTests)"/>
                        </xsl:call-template>
                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                    </td>
                    <xsl:if test="($report.test_suites_only) = 'false'">
                        <td width="11%" align="center" class="singleborder">
                            <xsl:choose>
                                <xsl:when test="($auth) = ''">
                                    <xsl:choose>
                                        <xsl:when test="($isFunctional) = 'true'">
                                            <xsl:value-of select="count(/ResultsSession/Exec/ExecViols/*[not(local-name() = 'ChangeImpact')][not(@supp='true')])"/>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:value-of select="count(/ResultsSession/Exec/ExecViols/ChangeImpact[not(@supp='true')])"/>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:choose>
                                        <xsl:when test="($isFunctional) = 'true'">
                                            <xsl:value-of select="count(/ResultsSession/Exec/ExecViols/*[not(local-name() = 'ChangeImpact')][@auth=($auth) and not(@supp='true')])"/>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:value-of select="count(/ResultsSession/Exec/ExecViols/ChangeImpact[@auth=($auth) and not(@supp='true')])"/>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </xsl:otherwise>
                            </xsl:choose>
                            <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                        </td>
                    </xsl:if>
                </tr>
            </xsl:if>

        <xsl:choose>
            <xsl:when test="($report.test_suites_only) = 'false'">
            <tr>
                <td align="left" class="singleborder right_separator" style="padding-left:5px;">
                   <xsl:if test="($report.test_suites_only) = 'true'">
                       <xsl:attribute name="colspan">5</xsl:attribute>
                   </xsl:if>
                   <xsl:if test="($report.test_suites_only) = 'false'">
                       <xsl:attribute name="colspan">6</xsl:attribute>
                   </xsl:if>
                   <table width="{$width}" cellpadding="0" cellspacing="0">
                       <xsl:if test="($report_format_mode='pdf')">
                           <xsl:attribute name="cols">89% 11%</xsl:attribute>
                       </xsl:if>
                       <xsl:for-each select="($topTest)">
                         
                           <xsl:call-template name="func_test_info">
                               <xsl:with-param name="test" select="." />
                               <xsl:with-param name="testID" select="./@id" />
                               <xsl:with-param name="depth">0</xsl:with-param>
                               <xsl:with-param name="auth" select="($auth)"/>
                               <xsl:with-param name="isFunctional" select="($isFunctional)"/>
                           </xsl:call-template>
                       </xsl:for-each>
                  </table>
               </td>
            </tr>
            </xsl:when>
            <xsl:otherwise>
                <xsl:for-each select="($topTest)">
                    <xsl:call-template name="func_test_info">
                        <xsl:with-param name="test" select="." />
                        <xsl:with-param name="testID" select="./@id" />
                        <xsl:with-param name="depth">0</xsl:with-param>
                        <xsl:with-param name="auth" select="($auth)"/>
                        <xsl:with-param name="isFunctional" select="($isFunctional)"/>
                    </xsl:call-template>
                </xsl:for-each>
            </xsl:otherwise>
            </xsl:choose>
        </table>
    </xsl:template>

    <!-- ================= summary_header ============================ -->
   <xsl:template name="summary_header">
      <xsl:param name="isFunctional" />
      <xsl:param name="useExpandCollapse" />

      <table width='{$width}' border='0' cellpadding='0'
         cellspacing='0'>
         <xsl:if test="($report_format_mode='pdf')">
            <xsl:attribute name="cols">15% 70% 15%</xsl:attribute>
         </xsl:if>

         <tr>
            <td width='20%'></td>
            <td align="center">
               <div class="small_header">
                  <xsl:choose>
                     <xsl:when test="($isFunctional) = 'false'">
                        <xsl:value-of select="$RES_PARAMETER_ASSET_SUMMARY" />
                     </xsl:when>
                     <xsl:when test="($toolName='Virtualize')">
                        <xsl:value-of
                           select="$RES_PARAMETER_ACTION_SUITE_SUMMARY" />
                     </xsl:when>
                     <xsl:otherwise>
                        <xsl:value-of
                           select="$RES_PARAMETER_TEST_SUITE_SUMMARY" />
                     </xsl:otherwise>
                  </xsl:choose>
               </div>
            </td>

            <td width='20%' class="collapse_controls" style='padding:10px;'>
               <xsl:call-template name="createStandardSectionControls">
                   <xsl:with-param name="useExpandCollapse" select="$useExpandCollapse"/>
                   <xsl:with-param name="expand_func_details" select="'expandCollapseAllDetails(true)'"/>
                   <xsl:with-param name="collapse_func_details" select="'expandCollapseAllDetails(false)'"/>
               </xsl:call-template>
            </td>
         </tr>
      </table>
   </xsl:template>

    <!-- =================  print a tests information  ============================ -->
    <xsl:template name="func_test_info">
        <xsl:param name="test" />
        <xsl:param name="testID" />
        <xsl:param name="depth" />
        <xsl:param name="auth" />
        <xsl:param name="isFunctional" />

        <xsl:variable name="width" select="($depth) * 20" />
        <xsl:variable name="widthPX" select="concat(($width),'px')" />
        <xsl:variable name="background" saxon:assignable="yes">singleborder</xsl:variable>
        <xsl:if test="($report.test_suites_only) = 'false'">    
           <saxon:assign name="background">no_error</saxon:assign>
        </xsl:if>
        <xsl:variable name="fontColor" saxon:assignable="yes">green</xsl:variable>

        <xsl:variable name="hasDetails" select="(($report.associations = 'true' and (count(./assoc) > 0 or (./Detail/@msg != ''))) or (count(.//Screenshot) > 0)) and (($isFunctional) = 'true')" />
        <xsl:variable name="detailsID" select="concat(($testID), '_details')" />

        <xsl:variable name="isFailed" saxon:assignable="yes">0</xsl:variable>
        <xsl:choose>
            <xsl:when test="($isFunctional) = 'true'">
                <saxon:assign name="isFailed" select="./@fail"/>
            </xsl:when>
            <xsl:otherwise>
                <saxon:assign name="isFailed" select="./@change"/>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:if test="($isFailed) > 0">
            <xsl:if test="($report.failed_tests_only) = 'false'">
                <xsl:choose>
                    <xsl:when test="($report.test_suites_only) = 'false'">    
                        <saxon:assign name="background">error</saxon:assign>
                    </xsl:when>
                    <xsl:otherwise>
                        <saxon:assign name="background">errorsingleborder</saxon:assign>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:if>
            <saxon:assign name="fontColor">red</saxon:assign>
        </xsl:if>
        <xsl:variable name="isSuite" saxon:assignable="yes"></xsl:variable>
        <xsl:for-each select="($test)">
            <saxon:assign name="isSuite" select="name() = 'TestSuite'" />
        </xsl:for-each>
        <xsl:choose>
            <xsl:when test="($isSuite) > 0 or ($depth) = 0">
                <xsl:for-each select="($test)">
                    <xsl:variable name="test_name">
                        <xsl:choose>
                            <xsl:when test="($depth) = 0 and ($isSuite) > 0 and ($report.test_suites_only) = 'true'">
                                <xsl:call-template name="parent_path_to_test">
                                    <xsl:with-param name="dirty_path" select="($testID)" />
                                </xsl:call-template>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:call-template name="func_test_name">
                                    <xsl:with-param name="includePath" select="($depth) = 0"/>
                                    <xsl:with-param name="name" select="@name"/>
                                    <xsl:with-param name="testID" select="($testID)"/>
                                    <xsl:with-param name="testNode" select="."/>
                                </xsl:call-template>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="totalSuccess">
                        <xsl:call-template name="total_success_sub_tests">
                            <xsl:with-param name="suite" select="." />
                            <xsl:with-param name="auth" select="($auth)" />
                            <xsl:with-param name="isFunctional" select="($isFunctional)" />
                        </xsl:call-template>
                    </xsl:variable>
                    <xsl:variable name="totalTests">
                        <xsl:call-template name="total_sub_tests">
                            <xsl:with-param name="suite" select="." />
                            <xsl:with-param name="isFunctional" select="($isFunctional)" />
                        </xsl:call-template>
                    </xsl:variable>
                    <xsl:variable name="totalFailed">
                        <xsl:call-template name="total_failed_sub_tests">
                            <xsl:with-param name="suite" select="." />
                            <xsl:with-param name="auth" select="($auth)" />
                            <xsl:with-param name="isFunctional" select="($isFunctional)" />
                        </xsl:call-template>
                    </xsl:variable>
                    <xsl:variable name="failed" select="($totalTests) - ($totalSuccess)"/>
                    <xsl:if test="($failed) > 0 and ($report.failed_tests_only) = 'false'">
                    <xsl:choose>
                        <xsl:when test="($report.test_suites_only) = 'false'">    
                            <saxon:assign name="background">error</saxon:assign>
                        </xsl:when>
                        <xsl:otherwise>
                            <saxon:assign name="background">errorsingleborder</saxon:assign>
                        </xsl:otherwise>
                    </xsl:choose>
                    </xsl:if>
                    <xsl:if test="($failed) > 0 or ($report.failed_tests_only) = 'false'">
                        
                            <xsl:choose>
                                <xsl:when test="($report.test_suites_only) = 'false'">
                                <tr>
                                    <td align="left" colspan="2" class='testSuiteTree' style="padding-left:{$widthPX}">
                                        <xsl:if test="($depth) = 0 and ($isSuite) = 0">
                                            <xsl:attribute name="class">singleborder right_separator</xsl:attribute>
                                        </xsl:if>
                                        <xsl:if test="($report_format_mode='pdf')">
                                           <xsl:call-template name="make_indentation">
                                              <xsl:with-param name="indentation" select="($depth)*4"/>
                                           </xsl:call-template>
                                        </xsl:if>
                                        <b><xsl:value-of select="($test_name)" /></b>
                                        <b>
                                            <xsl:text> [</xsl:text>
                                            <xsl:choose>
                                                <xsl:when test="($isFunctional) = 'true'">
                                                    <xsl:value-of select="($totalSuccess)"/>
                                                </xsl:when>
                                                <xsl:otherwise>
                                                    <xsl:value-of select="($totalFailed)"/>
                                                </xsl:otherwise>
                                            </xsl:choose>
                                            <xsl:text>/</xsl:text>
                                            <xsl:value-of select="($totalTests)"/>
                                            <xsl:text>]</xsl:text>
                                        </b>
                                    </td>
                                </tr>    
                                </xsl:when>
                                <xsl:otherwise>
                                <tr class="{$background} alternating">
                                    <td align="left" style="padding-left:{$widthPX};"
                                        class="singleborder right_separator">
                                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                                        <xsl:if test="($report_format_mode='pdf')">
                                           <xsl:call-template name="make_indentation">
                                              <xsl:with-param name="indentation" select="($depth)*4"/>
                                           </xsl:call-template>
                                        </xsl:if>
                                        <xsl:value-of select="($test_name)" />
                                    </td>
                                    <td align="center" class="singleborder">
                                        <xsl:value-of select="($totalFailed)"/>
                                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                                    </td>
                                    <td align="center" class="singleborder">
                                        <xsl:value-of select="($totalSuccess)"/>
                                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                                    </td>
                                    <td align="center" class="singleborder">
                                        <xsl:value-of select="($totalTests)"/>
                                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                                    </td>
                                    <td align="center" class="singleborder">
                                        <xsl:variable name="numerator">
                                            <xsl:choose>
                                                <xsl:when test="($isFunctional) = 'true'">
                                                    <xsl:value-of select="($totalSuccess)" />
                                                </xsl:when>
                                                <xsl:otherwise>
                                                    <xsl:value-of select="($totalFailed)" />
                                                </xsl:otherwise>
                                            </xsl:choose>
                                        </xsl:variable>
                                        <xsl:call-template name="get_percent">
                                            <xsl:with-param name="numerator" select="($numerator)"/>
                                            <xsl:with-param name="denominator" select="($totalTests)"/>
                                        </xsl:call-template>
                                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                                    </td>
                                </tr>
                                </xsl:otherwise>
                            </xsl:choose>
                    </xsl:if>
                </xsl:for-each>
            </xsl:when>
            <xsl:otherwise>
                <xsl:if test="($report.test_suites_only) = 'false'">
                    <xsl:if test="($isFailed) > 0 or ($report.failed_tests_only) = 'false'">
                        <xsl:variable name="associatedErrors">
                            <xsl:choose>
                                <xsl:when test="($auth) = ''">
                                    <xsl:choose>
                                        <xsl:when test="($isFunctional) = 'true'">
                                            <xsl:value-of select="count(/ResultsSession/Exec/ExecViols/*[not(local-name() = 'ChangeImpact')][@testId=($testID) and not(@supp='true')])" />
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:value-of select="count(/ResultsSession/Exec/ExecViols/ChangeImpact[@testId=($testID) and not(@supp='true')])" />
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:choose>
                                        <xsl:when test="($isFunctional) = 'true'">
                                            <xsl:value-of select="count(/ResultsSession/Exec/ExecViols/*[not(local-name() = 'ChangeImpact')][@testId=($testID) and @auth=($auth) and not(@supp='true')])" />
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:value-of select="count(/ResultsSession/Exec/ExecViols/ChangeImpact[@testId=($testID) and @auth=($auth) and not(@supp='true')])" />
                                        </xsl:otherwise>
                                    </xsl:choose>
                                    
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>

                        <tr class="{$background}singleborder">
                            <td align="left" class="right_separator testSuiteTree" style="padding-left:{$widthPX}">
                                <xsl:if test="($report_format_mode='pdf')">
                                    <xsl:call-template name="make_indentation">
                                       <xsl:with-param name="indentation" select="($depth)*4"/>
                                    </xsl:call-template>
                                </xsl:if>
                                <xsl:for-each select="($test)">
                                    <xsl:if test="($hasDetails) and (not(($report_type) = ($email_report)))">
                                        <span onclick="toggleTestDetails('{$detailsID}', this)" class="toggle_button" >[-]</span>
                                    </xsl:if>
                                    <xsl:value-of select="@name" />
                                </xsl:for-each>
                            </td>
                            <td align="center" width='10%' class='testSuiteTree'>
                                <font class="{$fontColor}">
                                    <b><xsl:value-of select="($associatedErrors)" /><xsl:value-of select="($nbsp)" disable-output-escaping="yes"/></b>
                                </font>
                            </td>
                        </tr>
                        <xsl:if test="($hasDetails)">
                            <xsl:call-template name="print_test_summary_details">
                                <xsl:with-param name="comment" select="./Detail/@msg" />
                                <xsl:with-param name="screenshots" select=".//Screenshot" />
                                <xsl:with-param name="detailsID" select="($detailsID)" />
                                <xsl:with-param name="depth" select="($depth) + 1" />
                            </xsl:call-template>
                        </xsl:if>
                    </xsl:if>
                </xsl:if>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:if test="($report.test_suites_only) = 'false'">
            <xsl:for-each select="($test)">
                <xsl:choose>
                    <xsl:when test="$isFunctional = 'true'">
                        <xsl:for-each select="./Test[@total > 0] | ./TestSuite[@total > 0]">
                            <xsl:call-template name="func_test_info">
                                <xsl:with-param name="test" select="." />
                                <xsl:with-param name="testID" select="./@id" />
                                <xsl:with-param name="auth" select="($auth)"/>
                                <xsl:with-param name="depth" select="($depth) + 1" />
                                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
                            </xsl:call-template>
                        </xsl:for-each>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:for-each select="./Test[@changeTotal > 0] | ./TestSuite[@changeTotal > 0]">
                            <xsl:call-template name="func_test_info">
                                <xsl:with-param name="test" select="." />
                                <xsl:with-param name="testID" select="./@id" />
                                <xsl:with-param name="auth" select="($auth)"/>
                                <xsl:with-param name="depth" select="($depth) + 1" />
                                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
                            </xsl:call-template>
                        </xsl:for-each>
                    </xsl:otherwise>
                </xsl:choose>
            

            </xsl:for-each>
        </xsl:if>
    </xsl:template>

    <!-- ==== print detail information for a single test under the test summary ==== -->
    <xsl:template name="print_test_summary_details">
        <xsl:param name="comment" />
        <xsl:param name="screenshots" />
        <xsl:param name="detailsID" />
        <xsl:param name="depth" />

        <xsl:variable name="width" select="($depth) * 20" />
        <xsl:variable name="widthPX" select="concat(($width),'px')" />

        <tr class="details_container" id="{$detailsID}">
            <td colspan="2" class="suite_details">
                <div class="suite_details" style="margin-left:{$widthPX}">

                     <table border="0" cellpadding="0" cellspacing="0">
                        <xsl:if test="($report_format_mode='pdf')">
                              <xsl:if test="($depth &lt; 2)">
                              <xsl:attribute name="cols">2% 98%</xsl:attribute>
                           </xsl:if>
                              <xsl:if test="($depth = 2)">
                              <xsl:attribute name="cols">3% 97%</xsl:attribute>
                           </xsl:if>
                              <xsl:if test="($depth = 3)">
                              <xsl:attribute name="cols">4% 96%</xsl:attribute>
                           </xsl:if>
                              <xsl:if test="($depth = 4)">
                              <xsl:attribute name="cols">5% 95%</xsl:attribute>
                           </xsl:if>
                              <xsl:if test="($depth = 5)">
                              <xsl:attribute name="cols">6% 94%</xsl:attribute>
                           </xsl:if>
                              <xsl:if test="($depth > 5)">
                              <xsl:attribute name="cols">7% 93%</xsl:attribute>
                           </xsl:if>
                        </xsl:if>
                          <tr>
                        <xsl:if test="($report_format_mode='pdf')">
                           <td></td>
                        </xsl:if>

                        <xsl:if test="not($report_format_mode='pdf')">
	                        <xsl:call-template name="print_screenshots">
	                           <xsl:with-param name="screenshots" select="($screenshots)"/>
	                           <xsl:with-param name="id" select="($detailsID)"/>
	                        </xsl:call-template>
	                    </xsl:if>
                        <xsl:if test="(($report.associations = 'true') and (($comment) != '' or count(./assoc) > 0))">
                            <td class="associations">

                                <xsl:call-template name="associations_line">
                                 <xsl:with-param name="isAssocNameBolded">true</xsl:with-param>
                                 <xsl:with-param name="areAssocsInSeparateLine">true</xsl:with-param>
                                </xsl:call-template>

                                <xsl:if test="($comment) != ''">
                                    <xsl:if test="count(./assoc) > 0">
                                       <br/>
                                    </xsl:if>
                                    <b><xsl:value-of select="$RES_PARAMETER_COMMENT"/></b><br/>
                                    <div><xsl:value-of select="($comment)"></xsl:value-of></div>
                                </xsl:if>
                            </td>
                        </xsl:if>
                    </tr>
                    </table>
                </div>
            </td>
        </tr>
    </xsl:template>

    <!-- ================== print the screenshot(s) ================================ -->
    <xsl:template name="print_screenshots">
       <xsl:param name="screenshots" />
       <xsl:param name="id" />

       <xsl:choose>
           <xsl:when test="count(($screenshots)) > 1">
               <xsl:variable name="screenshotCount" select="count(($screenshots))" />
                   <xsl:variable name="mul_img">multiple_window.png</xsl:variable>
                   <xsl:variable name="img_uri" select="concat(($output_dir),$mul_img)"/>
                   <xsl:variable name="img_url" select="translate($img_uri, '\','/')"/>
                   <xsl:variable name="res_img" saxon:assignable="yes"></xsl:variable>
                   <saxon:assign name="res_img" select="graph:publish($img_url,$mul_img)"/>
               <xsl:for-each select="(.//Screenshot)[1]"> <!-- put this in a template -->
                   <td>
                       <table>
                           <tr><td>
                                <div onclick="popupScreenshots('{$id}')" onmouseover="this.style.cursor='pointer';">
                                    <img class="screenshot" style="border:0" src="{$mul_img}" />
                                </div>
                           </td></tr>
                           <tr><td>
                                <a class="multiScreenshotLink" href="#" onclick="popupScreenshots('{$id}'); return false;">View screenshots (<xsl:value-of select="($screenshotCount)"/> in total)</a>
                           </td></tr>
                       </table>
                   </td>
               </xsl:for-each>
               <script language="JavaScript">
                   <xsl:for-each select="($screenshots)">
                       <xsl:variable name="width" select="./@width"/>
                       <xsl:variable name="height" select="./@height"/>
                       <xsl:variable name="url" select="./@url"/>
                       <xsl:variable name="iteration" select="./parent::FuncResultDetails/parent::TestCase/@id"/>
                       <xsl:variable name="code">storeScreenshotData(
                            '<xsl:value-of select="($id)"/>',
                            '<xsl:value-of select="($url)"/>',
                            '<xsl:value-of select="($iteration)"/>',
                            <xsl:value-of select="($width)"/>,
                            <xsl:value-of select="($height)"/>);
                       </xsl:variable>
                       <xsl:value-of select="($code)"/>
                   </xsl:for-each>
               </script>
           </xsl:when>
           <xsl:otherwise>
               <xsl:for-each select="($screenshots)">
                   <xsl:variable name="width" select="./@width" />
                   <xsl:variable name="height" select="./@height" />
                   <xsl:variable name="url" select="./@url" />
                   <td>
                       <div onclick="popupScreenshot('{$url}')" onmouseover="this.style.cursor='pointer';">
                           <img class="screenshot" width="{$width}" height="{$height}" src="{$url}" />
                       </div>
                   </td>
               </xsl:for-each>
           </xsl:otherwise>
       </xsl:choose>
    </xsl:template>

    <!-- ================== format the associations ================================ -->
    <xsl:template name="format_association_into_line">
        <xsl:param name="associations" />
        
        <xsl:variable name="line" select="substring-before($associations, ';')" />
        <xsl:choose>
            <xsl:when test="$line != ''">
                <xsl:call-template name="format_association_line">
                    <xsl:with-param name="line" select="$line" />
                </xsl:call-template>
                <xsl:variable name="next_line" select="substring-after($associations, ';')" />
                <xsl:if test="$next_line != ''">
                   <xsl:call-template name="format_association_into_line">
                       <xsl:with-param name="associations" select="$next_line"/>
                   </xsl:call-template>
                </xsl:if>
            </xsl:when>
            <xsl:when test="$associations != ''">
                <xsl:call-template name="format_association_line">
                    <xsl:with-param name="line" select="$associations" />
                </xsl:call-template>
            </xsl:when>
        </xsl:choose>
    </xsl:template>
    
    <!-- ================== format a single association name/value pair ============ -->
    <xsl:template name="format_association_line">
        <xsl:param name="line" />
        
        <xsl:variable name="name" select="substring-before($line, ':')" />
        <xsl:variable name="value" select="substring-after($line, ':')" />
        <b><xsl:value-of select="$name" /><xsl:text>:</xsl:text></b><xsl:value-of select="$value" /><br/>
    </xsl:template>

    <!-- ================== get percentage value =================================== -->
    <xsl:template name="get_percent">
        <xsl:param name="numerator"/>
        <xsl:param name="denominator"/>
        
        <xsl:choose>
            <xsl:when test="($denominator) = 0">
                <xsl:text>0</xsl:text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="floor((($numerator) div ($denominator))*100)"/>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:text>%</xsl:text>
    </xsl:template>

    <!-- =================  print details of functional errors ====================== -->
    <xsl:template name="func_summary_details">
       <xsl:param name="auth"/>
       <xsl:param name="authid"/>
       <xsl:param name="isFunctional"/>

       <xsl:variable name="author_func_history" select="count(/ResultsSession/Exec/History/Graph[@id=$func_tasks_graph_id]/Authors/Author[@name = $auth]/X)"/>

       <!-- functional history graph -->
       <xsl:if test="($generate_graphs = 'true') and (($author_func_history) > 0) and (($isFunctional) = 'true')">
          <tr><td colspan="3">
          <xsl:call-template name="generate_graph">
             <xsl:with-param name="graph" select="/ResultsSession/Exec/History/Graph[@id=$func_tasks_graph_id]"/>
             <xsl:with-param name="auth" select="$auth"/>
             <xsl:with-param name="img_name" select="concat('func_',$authid,'_',$org_time)"/>
          </xsl:call-template>
          <br/>
          </td></tr>
       </xsl:if>
       <xsl:if test="not($report.mail.compact = $trends) or not(($report_type) = ($email_report))">
       <xsl:call-template name="func_summary_details_by_test">
          <xsl:with-param name="auth"  select="$auth"/>
          <xsl:with-param name="authid"  select="$authid"/>
          <xsl:with-param name="isFunctional" select="($isFunctional)"/>
       </xsl:call-template>
       </xsl:if>
    </xsl:template>

    <!-- =================  details for each test case ============================== -->
    <xsl:template name="func_test_details">
        <xsl:param name="topTest" />
        <xsl:param name="auth" />
        <xsl:param name="isFunctional"/>
        <xsl:param name="testIdentifier"/>

            <xsl:variable name="testID" select="$topTest/@id" />
            <xsl:variable name="associatedErrors" saxon:assignable="yes"/>
            <xsl:choose>
                <xsl:when test="($isFunctional) = 'true'">
                    <saxon:assign name="associatedErrors" select="count(/ResultsSession/Exec/ExecViols/*[not(local-name() = 'ChangeImpact')][@auth=($auth) and @testId=($testID)])"/>
                </xsl:when>
                <xsl:otherwise>
                    <saxon:assign name="associatedErrors" select="count(/ResultsSession/Exec/ExecViols/ChangeImpact[@auth=($auth) and @testId=($testID)])"/>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:if test="(($associatedErrors) > 0)">
                <tr>
                    <td style="padding-left:5px;padding-right:5px" colspan="2" class="tree">
                        <b>
                            <xsl:call-template name="func_test_name">
                                <xsl:with-param name="includePath" select="true()"/>
                                <xsl:with-param name="name" select="@name"/>
                                <xsl:with-param name="testID" select="($testID)"/>
                                <xsl:with-param name="testNode" select="."/>
                            </xsl:call-template>
                        </b>
                    </td>
                </tr>
                <xsl:variable name="allEntries" saxon:assignable="yes" />
                <xsl:choose>
                    <xsl:when test="($isFunctional) = 'true'">
                        <saxon:assign name="allEntries" select="/ResultsSession/Exec/ExecViols/*[not(local-name() = 'ChangeImpact')][@auth=($auth) and @testId=($testID)]"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <saxon:assign name="allEntries" select="/ResultsSession/Exec/ExecViols/ChangeImpact[@auth=($auth) and @testId=($testID)]"/>
                    </xsl:otherwise>
                </xsl:choose>
                <xsl:for-each select="($allEntries)">
                    <xsl:variable name="violationId">
                        <xsl:value-of select="($testIdentifier)" />:<xsl:value-of select="position()" />
                    </xsl:variable>
                    <xsl:variable name="rule" select="@rule" />
                    <xsl:variable name="sev" select="@sev" />

                    <tr>
                        <td style="padding-left:15px" class="tree">
                            <xsl:variable name="details" select="@violationDetails"/>
                            <xsl:choose>
                                <xsl:when test="($details) and not(($report_type) = ($email_report))">
                                    <xsl:element name="a">
                                    <xsl:attribute name="class">blue_link</xsl:attribute>
                                    <xsl:attribute name="href">#</xsl:attribute>
                                    <xsl:attribute saxon:disable-output-escaping="yes" name="onclick">showDetails('<xsl:value-of select="$time"/>', 'details_<xsl:value-of select="$violationId"/>'); return false;</xsl:attribute>
                                         <xsl:value-of select="@msg" />
                                         <xsl:element name="input">
                                         <xsl:attribute name="type">hidden</xsl:attribute>
                                         <xsl:attribute saxon:disable-output-escaping="yes" name="id">details_<xsl:value-of select="$violationId"/></xsl:attribute>
                                         <xsl:attribute name="value"><xsl:value-of select="$details"/></xsl:attribute>
                                        </xsl:element>
                                    </xsl:element>

                                    <xsl:variable name="rev" select="@rev"/>
                                    <xsl:if test="(string-length($rev) > 0) and not($rev = 'unknown.revision')">
                                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                                        <font class="gray"><xsl:value-of select="$RES_PARAMETER_REVISION"/></font>
                                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                                        <font class="gray"><xsl:value-of select="($rev)"/></font>
                                    </xsl:if>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of select="@msg" />
                                    <xsl:variable name="rev" select="@rev"/>
                                    <xsl:if test="(string-length($rev) > 0) and not($rev = 'unknown.revision')">
                                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                                        <font class="gray"><xsl:value-of select="$RES_PARAMETER_REVISION"/></font>
                                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                                        <font class="gray"><xsl:value-of select="($rev)"/></font>
                                    </xsl:if>
                                </xsl:otherwise>
                            </xsl:choose>
                        </td>
                        <td style="padding-right:10px" align="right" class="tree">
                            <xsl:variable name="violTfcRef" select="@tfcRef" />
                            <xsl:variable name="trafficData" select="/ResultsSession/Exec/TrafficData/Traffic[@tfcRef=($violTfcRef)]" />

                            <xsl:if test="not(($report_type) = ($email_report))">
                                <xsl:choose>
                                    <xsl:when test="($trafficData)">
                                        <xsl:call-template name="showTraffic">
                                            <xsl:with-param name="trafficRef" select="($violTfcRef)" />
                                        </xsl:call-template>
                                    </xsl:when>
                                </xsl:choose>
                            </xsl:if>
                        </td>
                    </tr>
                    <xsl:call-template name="insure_stack_trace"/>
                </xsl:for-each>
            </xsl:if>
    </xsl:template>

    <xsl:template name="showTraffic">
        <xsl:param name="trafficRef" />
        <xsl:element name="a">
            <xsl:attribute name="href">#</xsl:attribute>
            <xsl:attribute saxon:disable-output-escaping="yes"
                name="onclick">showTrafficLog(['<xsl:value-of select="$trafficRef" />'],'<xsl:value-of select="$time"/>'); return false;</xsl:attribute>
            <xsl:attribute name="class">blue_link</xsl:attribute>
            <xsl:value-of select="($RES_PARAMETER_VIEW_TRAFFIC)" />
        </xsl:element>
    </xsl:template>

    <!-- =================  func summary_details_by_file ============================ -->
    <xsl:template name="func_summary_details_by_test">
        <xsl:param name="auth" />
        <xsl:param name="authid" />
        <xsl:param name="isFunctional" />

        <xsl:variable name="authErrors" select="tmp" saxon:assignable="yes" />
        <xsl:choose>
            <xsl:when test="($isFunctional) = 'true'">
                <saxon:assign name="authErrors"
                    select="/ResultsSession/Exec/ExecViols/*[not(local-name() = 'ChangeImpact')][@auth=($auth) and not(@supp='true')]" />
            </xsl:when>
            <xsl:otherwise>
                <saxon:assign name="authErrors"
                    select="/ResultsSession/Exec/ExecViols/ChangeImpact[@auth=($auth) and not(@supp='true')]" />
            </xsl:otherwise>
        </xsl:choose>

        <xsl:variable name="totErrs" select="count($authErrors)" />

        <xsl:if test="($totErrs > 0)">
            <tr>
                   <xsl:variable name="message" select="$RES_PARAMETER_TOTAL_CHANGES" saxon:assignable="yes"/>
                   <xsl:variable name="anchor" saxon:assignable="yes" select="concat('change_',$authid)"/>
                <xsl:if test="($isFunctional) = 'true'">
                    <saxon:assign name="anchor" select="concat('func_',$authid)"/>
                    <saxon:assign name="message" select="$RES_PARAMETER_TOTAL_FINDINGS"/>
                   </xsl:if>
                <xsl:call-template name="details_table_auth_header">
                    <xsl:with-param name="auth" select="$auth"/>
                    <xsl:with-param name="number" select="$totErrs"/>
                    <xsl:with-param name="message" select="$message"/>
                    <xsl:with-param name="colspan" select="2"/>
                    <xsl:with-param name="anchor" select="$anchor"/>
                     </xsl:call-template>
            </tr>
            <tr><td colspan="3">
                <xsl:call-template name="print_header_separator"/>
            </td></tr>
        </xsl:if>

        <xsl:variable name="tmp_file_name" select="empty"
            saxon:assignable="yes" />

        <!-- by test suite... -->
        <tr>
            <td align="left" colspan="3" >
                <table class="doubleborder" aligh="center" width="100%" cellpadding='0' cellspacing='0'>
                <xsl:if test="($report_format_mode='pdf')">
                <xsl:attribute name="cols">70% 30%</xsl:attribute>
            </xsl:if>

        <xsl:for-each select="/ResultsSession/Exec/ExecutedTestsDetails//Test">
            <xsl:call-template name="func_test_details">
                <xsl:with-param name="topTest" select="." />
                <xsl:with-param name="auth" select="($auth)" />
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
                <xsl:with-param name="testIdentifier" select="position()"/>
            </xsl:call-template>
        </xsl:for-each>
            </table>

            <xsl:if test="not(($report_type) = ($email_report)) and not($report_format_mode='pdf')">
                 <xsl:variable name="trafficIds" select="$authErrors/@tfcRef" />
                 <xsl:variable name="testTrafficCount" select="count($trafficIds)" />
                 <xsl:if test="($testTrafficCount > 0)">
                    <xsl:call-template name="add_traffic_data" >
                         <xsl:with-param name="trafficIds" select="$trafficIds"/>
                    </xsl:call-template>
                </xsl:if>
            </xsl:if>

            </td>
        </tr>

        <xsl:if test="($totErrs > 0)">
            <tr>
                <td colspan="3">
                    <br />
                </td>
            </tr>
        </xsl:if>

    </xsl:template>

    <!-- =================  functional tests auth list ============================ -->
    <xsl:template name="func_author_list">    
        <xsl:param name="isFunctional" />
        <xsl:for-each select="/ResultsSession/Authors/Author">
    
            <xsl:call-template name="func_author_info">
                <xsl:with-param name="auth" select="@name"/>
                <xsl:with-param name="authid" select="@id"/>
                <xsl:with-param name="authidx" select="position()"/>
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            </xsl:call-template>

       </xsl:for-each> 
    </xsl:template>

    <!-- =================  functional tests auth info ============================ -->
    <xsl:template name="func_author_info">
        <xsl:param name="auth" />
        <xsl:param name="authid" />
        <xsl:param name="authidx" />
        <xsl:param name="isFunctional" />

        <xsl:variable name="author_cs_violations"
            select="count(/ResultsSession/CodingStandards/StdViols/*[@auth=($auth) and not(@supp='true')])" />
        <xsl:variable name="author_change_impacts"
            select="count(/ResultsSession/Exec/ExecViols/ChangeImpact[@auth=($auth) and not(@supp='true')])" />
        <xsl:variable name="author_func_violations"
            select="count(/ResultsSession/Exec/ExecViols/*[not(local-name() = 'ChangeImpact')][@auth=($auth) and not(@supp='true')])" />
        <xsl:variable name="author_exec_violations"
            select="count(/ResultsSession/Exec/ExecViols/ExecViol[@auth=($auth)])" />
        <xsl:variable name="author_tcases"
            select="count(/ResultsSession/Generation/GenerationInformations/GenerationInfo[@auth=($auth)]/TestCaseList/TestCase)" />
        <xsl:variable name="author_recommended_errors"
            select="count(/ResultsSession/CodingStandards/StdViols/*[@auth=($auth) and not(@supp='true') and @urgent='true'])" />
            
        <xsl:variable name="author_all_tasks"
            select="($author_cs_violations) + ($author_tcases) + ($author_exec_violations) + ($author_func_violations) + ($author_change_impacts)" />

        <xsl:variable name="link_class" saxon:assignable="yes">blue_link</xsl:variable>
        <xsl:variable name="font_class" saxon:assignable="yes">black</xsl:variable>

        <!--xsl:if test="(($author_func_violations = 0) and ($isFunctional = 'true')) or (($author_change_impacts = 0) and ($isFunctional = 'false'))">
            <saxon:assign name="link_class">green_link</saxon:assign>
            <saxon:assign name="font_class">green</saxon:assign>
        </xsl:if-->
        <tr class="alternating">
            <td class="singleborder right_separator" align="left">
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
                <xsl:if test="($author_all_tasks) > 0">
                    <xsl:variable name="prefix_type" saxon:assignable="yes"/>
                    <xsl:choose>
                        <xsl:when test="($isFunctional) = 'true'">
                            <saxon:assign name="prefix_type">func_</saxon:assign>
                        </xsl:when>
                        <xsl:otherwise>
                            <saxon:assign name="prefix_type">change_</saxon:assign>
                        </xsl:otherwise>
                    </xsl:choose>
                    <xsl:if test="(($report.developer_reports) = 'true') and not(($report_type) = ($email_report))">

                        <xsl:variable name="author_html"
                            select="concat($dev_reports_prefix,$authid,'.html#',$prefix_type,$authid)" />
                        <xsl:variable name="onclick" saxon:assignable="yes"
                            select="concat('openWin(',$qt,$author_html,$qt,')')" />
                        <xsl:if test="($report_type) = ($tcm_report)">
                           <xsl:variable name="dev_rep_name" select="concat($dev_reports_prefix,$authid,'.html')"/>
                           <saxon:assign name="onclick" select="concat('openDevReport(',$qt,$dev_rep_name,$qt,')')"/>
                        </xsl:if>
                        <xsl:call-template name="standard_link_ref">
                            <xsl:with-param name="href"
                                select="'#'" />
                            <xsl:with-param name="onclick"
                                select="($onclick)" />
                            <xsl:with-param name="label"
                                select="($auth)" />
                            <xsl:with-param name="class"
                                select="($link_class)" />
                        </xsl:call-template>
                    </xsl:if>
                    <xsl:if
                        test="(($report.developer_errors) = 'true') and ((($report.developer_reports) = 'false') or (($report_type) = ($email_report)))">
                        <a class="{$link_class}" href="#{$prefix_type}{$authid}">
                            <xsl:value-of select="($auth)" />
                        </a>
                    </xsl:if>
                    <xsl:if
                        test="(($report.developer_errors) = 'false') and (($report.developer_reports = 'false') or (($report_type) = ($email_report)))">
                        <font class="{$font_class}">
                            <xsl:value-of select="($auth)" />
                        </font>
                    </xsl:if>
                </xsl:if>
                <xsl:if test="not(($author_all_tasks) > 0)">
                    <font class="{$font_class}">
                        <xsl:value-of select="($auth)" />
                    </font>
                </xsl:if>
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
            </td>
            <!-- NYI Functional Suppressions
            <td align="center">
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
                <xsl:value-of select="($author_cs_supp)" />
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
            </td> -->
            <!-- NYI Functional Test history
            <xsl:if
                test="count(/ResultsSession/Exec/History/Author) > 0">
                <td align="center">
                    <xsl:value-of select="($nbsp)"
                        disable-output-escaping="yes" />

                    <xsl:call-template
                        name="format_author_total_delta">
                        <xsl:with-param name="delta"
                            select="/ResultsSession/Exec/History/Author[@name=($auth)]/Delta" />
                    </xsl:call-template>

                    <xsl:value-of select="($nbsp)"
                        disable-output-escaping="yes" />
                </td>
            </xsl:if> -->
            <td class="singleborder" align="center">
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
                <xsl:choose>
                    <xsl:when test="($isFunctional) = 'true'">
                        <xsl:value-of select="($author_func_violations)" />
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="($author_change_impacts)" />
                    </xsl:otherwise>
                </xsl:choose>
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
            </td>
            <td class="singleborder" align="center">
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
                <xsl:choose>
                    <xsl:when test="($isFunctional) = 'true'">
                         <xsl:value-of select="results:getTotalValue(/ResultsSession/Exec/ExecutedTestsDetails/Total/@authFail,$authidx)" />
                    </xsl:when>
                    <xsl:otherwise>
                         <xsl:value-of select="results:getTotalValue(/ResultsSession/Exec/ExecutedTestsDetails/Total/@authChange,$authidx)" />
                    </xsl:otherwise>
                </xsl:choose>
               
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
                <!-- xsl:call-template name="total_failed_tests">
                    <xsl:with-param name="auth" select="($auth)"/>
                </xsl:call-template-->
            </td>
            <!-- NYI Recommended errors
            <td align="center">
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
                <xsl:value-of select="($author_recommended_errors)" />
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
            </td> -->
        </tr>
        <!-- TODO remove? /xsl:if -->
    </xsl:template>
    
    <!-- =================  print functional error details for each author ===== -->
    <xsl:template name="func_error_details">
        <xsl:param name="isFunctional"/>
       <xsl:for-each select="/ResultsSession/Authors/Author">
            <br />
            <table align="center" class="singleborder" width="{$authors_width}"
                cellpadding='0' cellspacing='0'>
                <xsl:if test="($report_format_mode='pdf')">
                    <xsl:attribute name="cols">33% 33% 34%</xsl:attribute>
                </xsl:if>
                <xsl:call-template name="func_summary_details_by_test">
                    <xsl:with-param name="auth" select="@name"/>
                    <xsl:with-param name="authid" select="@id"/>
                    <xsl:with-param name="isFunctional" select="($isFunctional)"/>
                </xsl:call-template>
            </table>
       </xsl:for-each>
    </xsl:template>

    <!-- ==================  print result details with traffic ============================ -->
    <xsl:template name="func_test_result_details">
        <xsl:param name="topTest" />

        <xsl:variable name="testName" select="$topTest/@name" />
        <xsl:variable name="testId" select="@id" />
        <xsl:variable name="iterationWithTrafficCount" select="count(TestCase/@tfcRef)" />

        <tr>
            <td colspan="2" class="tree" style="padding-left:10px;">
                <xsl:call-template name="func_test_name">
                    <xsl:with-param name="includePath" select="true()"/>
                    <xsl:with-param name="name" select="$testName"/>
                    <xsl:with-param name="testID" select="@id"/>
                    <xsl:with-param name="testNode" select="."/>
                </xsl:call-template>
            </td>
            <td style="padding-right:10px;" align="right" nowrap="nowrap">
                <xsl:choose>
                    <xsl:when test="$iterationWithTrafficCount > 0 and (not(($report_type) = ($email_report)))">
                        <xsl:call-template name="showTrafficWithIterations">
                            <xsl:with-param name="Test" select="." />
                        </xsl:call-template>
                    </xsl:when>
                    <xsl:when test="@tfcRef">
                        <xsl:variable name="violTfcRef" select="@tfcRef" />
                        <xsl:variable name="trafficData" select="/ResultsSession/Exec/TrafficData/Traffic[@tfcRef=($violTfcRef)]" />
                
                        <xsl:call-template name="showTraffic">
                            <xsl:with-param name="trafficRef" select="$violTfcRef" />
                        </xsl:call-template>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="($RES_PARAMETER_RESULT_NA)" />
                    </xsl:otherwise>
                </xsl:choose>
            </td>
        </tr>
    </xsl:template>

    <xsl:template name="showTrafficWithIterations">
        <xsl:param name="Test" />

        <xsl:variable name="testId" select="($Test)/@id" />
        <xsl:variable name="apos">'</xsl:variable>
        <xsl:variable name="jsParams" saxon:assignable="yes">[</xsl:variable>

        <xsl:for-each select="TestCase">
            <xsl:if test="string-length($jsParams) > 2">
                <saxon:assign name="jsParams" select="concat($jsParams, ',')"/>
            </xsl:if>
            <saxon:assign name="jsParams" select="concat($jsParams, $apos, (@tfcRef), $apos)"/>
        </xsl:for-each>
        <saxon:assign name="jsParams" select="concat($jsParams, ']')"/>

        <a class="blue_link"
           href="#" onclick="showTrafficLog({$jsParams}, '{$time}'); return false;">
           <xsl:value-of select="($RES_PARAMETER_VIEW_TRAFFIC)" />
        </a>

    </xsl:template>

    <!-- ==================  print functional ============================ -->
    <xsl:template name="display_functional">
    
        <!-- print functional tests header -->
        <xsl:if test="($functional > 0) or ($change_advisor > 0)">

            <xsl:call-template name="summary_section_header">
               <xsl:with-param name="ref">func</xsl:with-param>
            
               <xsl:with-param name="id">functional_summary_section</xsl:with-param>
                <xsl:with-param name="label">
                        <xsl:choose>
                           <xsl:when test="($toolName='Virtualize')">
                               <xsl:value-of select="$RES_PARAMETER_SUMMARY_PROVISIONING_ACTIONS"/>
                           </xsl:when>
                           <xsl:otherwise>
                               <xsl:value-of select="$RES_PARAMETER_SUMMARY_FUNCTIONAL_TESTS" />
                           </xsl:otherwise>
                       </xsl:choose>
                </xsl:with-param>
            </xsl:call-template>

            <xsl:call-template name="func_summary_charts"/>

            <xsl:call-template name="display_apicoverage"/>
            <xsl:if test="($functional > 0)">
            <xsl:call-template name="collapsible_section_header">
                <xsl:with-param name="id">functional_details_section</xsl:with-param>
                <xsl:with-param name="label">
                        <xsl:choose>
                           <xsl:when test="($toolName='Virtualize')">
                               <xsl:value-of select="$RES_PARAMETER_DETAILS_PROVISIONING_ACTIONS"/>
                           </xsl:when>
                           <xsl:otherwise>
                               <xsl:value-of select="$RES_PARAMETER_DETAILS_FUNCTIONAL_TESTS" />
                           </xsl:otherwise>
                       </xsl:choose>
                </xsl:with-param>
            </xsl:call-template>

            <div id="functional_details_section">
            <br/>
            <xsl:call-template name="func_header">
                <xsl:with-param name="isFunctional" select="'true'" />
            </xsl:call-template>
            
                <xsl:if test="($generate_graphs = 'true')">
                   <xsl:if test="($func_tc_history) > 0">
                      <br/>
                      <xsl:call-template name="generate_graph">
                         <xsl:with-param name="graph" select="/ResultsSession/Exec/History/Graph[@id=$func_testcases_graph_id]"/>
                         <xsl:with-param name="img_name" select="concat('func_tc_',$org_time)"/>
                      </xsl:call-template>
                   </xsl:if>
                   <xsl:if test="($func_history) > 0">
                      <br/>
                      <xsl:call-template name="generate_graph">
                         <xsl:with-param name="graph" select="/ResultsSession/Exec/History/Graph[@id=$func_tasks_graph_id]"/>
                         <xsl:with-param name="img_name" select="concat('func_',$org_time)"/>
                      </xsl:call-template>    
                   </xsl:if>
                </xsl:if>

            <!-- functional tests summary -->
            <br/>
            <xsl:call-template name="func_tests_summary">
                <xsl:with-param name="auth"></xsl:with-param>
                <xsl:with-param name="isFunctional" select="'true'"/>
            </xsl:call-template>

            <xsl:if test="not($report.mail.compact = $trends) or not(($report_type) = ($email_report))">

            <xsl:if test="($report.authors_details) = 'true'">

                <br/>
                <xsl:if test="($func_errors_nmb > 0)">
                    <xsl:call-template name="func_authors_summary">
                        <xsl:with-param name="isFunctional" select="'true'" />
                    </xsl:call-template>
                </xsl:if>

                <!-- print functional errors for each author -->
                <xsl:if test="($report.developer_errors) = 'true'">
                    <xsl:call-template name="func_error_details">
                        <xsl:with-param name="isFunctional" select="'true'"/>
                    </xsl:call-template>
                </xsl:if>

            </xsl:if>

            <xsl:variable name="testTrafficCount" select="count(/ResultsSession/Exec/ExecutedTestsDetails//Test[@status='pass']/@tfcRef) + count(/ResultsSession/Exec/ExecutedTestsDetails//TestCase[@status='pass']/@tfcRef)" />

            <xsl:if test="($testTrafficCount > 0)">

                <table width='{$width}' border='0' bordercolor='{$func_bdrcolor}'
                    cellpadding='0' cellspacing='0'>
                    <xsl:if test="($report_format_mode='pdf')">
                        <xsl:attribute name="cols">33% 33% 34%</xsl:attribute>
                    </xsl:if>
                    <tr>
                        <td colspan="3">
                           <xsl:call-template name="test_traffic_header" />
                        </td>
                    </tr>
                    <xsl:for-each select="/ResultsSession/Exec/ExecutedTestsDetails//Test">
                        <xsl:call-template name="func_test_result_details">
                            <xsl:with-param name="topTest" select="." />
                        </xsl:call-template>
                    </xsl:for-each>
                </table>

            </xsl:if>

            <xsl:if test="not(($report_type) = ($email_report)) and not($report_format_mode='pdf')">
                 <xsl:if test="($testTrafficCount > 0) or (($report.authors_details) = 'true' and ($report.developer_errors) = 'true')">
                    <xsl:call-template name="add_traffic_data"/>
                </xsl:if>
            </xsl:if>

            </xsl:if>
            </div>
            </xsl:if>
            <xsl:if test="($change_advisor > 0)">
              <xsl:call-template name="display_change_impact"/>
            </xsl:if>
        </xsl:if>
    </xsl:template>

    <!-- ==================  print change impact summary ============================ -->
    <xsl:template name="display_change_impact">

        <!-- print change impact tests header -->
            <xsl:call-template name="collapsible_section_header">
                <xsl:with-param name="id">change_impact_details_section</xsl:with-param>
                <xsl:with-param name="label" select="$RES_PARAMETER_DETAILS_CHANGE_IMPACTS"/>
            </xsl:call-template>

            <div id="change_impact_details_section">
            <br/>
            <xsl:call-template name="func_header">
                <xsl:with-param name="isFunctional" select="'false'" />
            </xsl:call-template>

            <!-- change impact tests summary -->
            <br/>
            <xsl:call-template name="func_tests_summary">
                <xsl:with-param name="auth"></xsl:with-param>
                <xsl:with-param name="isFunctional" select="'false'"/>
            </xsl:call-template>

            <xsl:if test="not($report.mail.compact = $trends) or not(($report_type) = ($email_report))">

            <xsl:if test="($report.authors_details) = 'true'">

                <br/>
                <xsl:if test="($change_impacts_nmb > 0)">
                    <xsl:call-template name="func_authors_summary">
                        <xsl:with-param name="isFunctional" select="'false'" />
                    </xsl:call-template>
                </xsl:if>

                <!-- print change impact errors for each author -->
                <xsl:if test="($report.developer_errors) = 'true'">
                    <xsl:call-template name="func_error_details">
                        <xsl:with-param name="isFunctional" select="'false'"/>
                    </xsl:call-template>
                </xsl:if>

            </xsl:if>

            <xsl:if test="not(($report_type) = ($email_report)) and not($report_format_mode='pdf')">
                 <xsl:if test="(($report.authors_details) = 'true' and ($report.developer_errors) = 'true')">
                    <xsl:call-template name="add_traffic_data"/>
                </xsl:if>
            </xsl:if>

            </xsl:if>
            </div>
    </xsl:template>

    <!-- =================  store traffic data inputs ============================ -->
    <xsl:template name="add_traffic_data">
        <!-- empty list means all traffic data -->
        <xsl:param name="trafficIds"/>

        <xsl:choose>
           <xsl:when test="$trafficIds">
              <xsl:variable name="idsList" saxon:assignable="yes"></xsl:variable>

              <xsl:for-each select="$trafficIds">

                 <xsl:variable name="trafficRef" select="." />
                 <xsl:variable name="trafficRefKey" select="concat($trafficRef,',')" />

                 <xsl:if test="not(contains($idsList, $trafficRefKey))">
                    <saxon:assign name="idsList" select="concat($idsList, $trafficRefKey)" />
                    <xsl:variable name="trafficData" select="/ResultsSession/Exec/TrafficData/Traffic[@tfcRef=($trafficRef)]" />
                    <xsl:call-template name="single_traffic_data" >
                         <xsl:with-param name="trafficData" select="$trafficData"/>
                    </xsl:call-template>
                 </xsl:if>
              </xsl:for-each>
           </xsl:when>
           <xsl:otherwise>
              <xsl:for-each select="/ResultsSession/Exec/TrafficData/Traffic">
                  <xsl:call-template name="single_traffic_data" >
                     <xsl:with-param name="trafficData" select="."/>
                  </xsl:call-template>
              </xsl:for-each>
           </xsl:otherwise>
       </xsl:choose>

    </xsl:template>

    <xsl:template name="single_traffic_data">
        <xsl:param name="trafficData"/>

        <xsl:variable name="trafficRef" select="$trafficData/@tfcRef" />
        <xsl:variable name="responseTrafficTime" select="$trafficData/@serverResTime" />
        <xsl:variable name="requestTrafficBody" select="$trafficData/@reqBody" />
        <xsl:variable name="responseTrafficBody" select="$trafficData/@resBody" />
        <xsl:variable name="requestTrafficHeaders" select="$trafficData/@reqHeaders" />
        <xsl:variable name="responseTrafficHeaders" select="$trafficData/@resHeaders" />
        <input type="hidden" id="reqHeader_{$trafficRef}" value="{$requestTrafficHeaders}" />
        <input type="hidden" id="reqBody_{$trafficRef}" value="{$requestTrafficBody}" />
        <input type="hidden" id="resHeader_{$trafficRef}" value="{$responseTrafficHeaders}" />
        <input type="hidden" id="resBody_{$trafficRef}" value="{$responseTrafficBody}" />
        <input type="hidden" id="serverResTime_{$trafficRef}" value="{$responseTrafficTime}" />
    </xsl:template>

    <!-- =================  test traffic (all tests) header ============================ -->
    <xsl:template name="test_traffic_header">

    <table width='{$width}' cellpadding='0' cellspacing='0'>
        <xsl:if test="($report_format_mode='pdf')">
            <xsl:attribute name="cols">20% 60% 20%</xsl:attribute>
        </xsl:if>
    <tr>
    <td width="20%"></td>
    <td width="60%" align="center"><div class="small_header"><xsl:value-of select="$RES_PARAMETER_TEST_TRAFFIC"/></div></td>
    <td width="20%" align="right">
       <xsl:if test="not($report_type = $email_report)">
         <xsl:call-template name="back_to_top"/>
       </xsl:if>
    </td>
    </tr>
    <tr><td colspan="3">
        <xsl:call-template name="print_header_separator"/>
    </td></tr>
    </table>

    </xsl:template>
        
</xsl:stylesheet>
